@@ -56,6 +56,7 @@ CLI_BUILTIN_DECL_STATIC(unregister);
 
 ZSV_MAIN_DECL(select);
 ZSV_MAIN_DECL(count);
+ZSV_MAIN_DECL(paste);
 ZSV_MAIN_DECL(2json);
 ZSV_MAIN_DECL(2tsv);
 ZSV_MAIN_DECL(serialize);
@@ -88,6 +89,7 @@ struct builtin_cmd builtin_cmds[] = {
 
   CLI_BUILTIN_COMMAND(select),
   CLI_BUILTIN_COMMAND(count),
+  CLI_BUILTIN_COMMAND(paste),
   CLI_BUILTIN_COMMAND(2json),
   CLI_BUILTIN_COMMAND(2tsv),
   CLI_BUILTIN_COMMAND(serialize),
